import java.io.*;
import java.util.*;

/**
 * Read in the values. Scan over the array. Test each entry to see if it is
 * the corner of an L. For each distinct integer, there must be a grid entry
 * that is the corner of an L. Additionally, make sure that each positive 
 * number is seen exactly 3 times.
 *
 * @author Finn Lidbetter
 */

public class ybtj2_finn {
  public static void main(String[] args) throws IOException {
    BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
    int n = Integer.parseInt(br.readLine());
    int nVals = ((1<<n)*(1<<n)-1)/3;
    int[][] grid = new int[1<<n][1<<n];
    int[] counts = new int[nVals+1];
    boolean[] hasL = new boolean[nVals+1];
    for (int i=0; i<1<<n; i++) {
      String[] s = br.readLine().split(" ");
      for (int j=0; j<1<<n; j++) {
        grid[i][j] = Integer.parseInt(s[j]);
      }
    }
    for (int r=0; r<grid.length; r++) {
      for (int c=0; c<grid[r].length; c++) {
        counts[grid[r][c]]++;
        if (isLCorner(grid, r, c)) {
          hasL[grid[r][c]] = true;
        }
      }
    }
    boolean good = true;
    for (int i=1; i<=nVals; i++) {
      if (counts[i]!=3 || !hasL[i]) {
        good = false;
        break;
      }
    }
    if (counts[0]!=1) {
      good = false;
    }
    if (good) 
      System.out.println(1);
    else
      System.out.println(0);
  }

  static boolean isLCorner(int[][] grid, int r, int c) {
    // Entry 1:  #.
    //           ##
    //
    // Entry 2:  ##
    //           #.
    //
    // Entry 3:  ##
    //           .#
    //
    // Entry 4:  .#
    //           ##
    int[] dr1 = new int[]{-1, 0, 1, 0};
    int[] dc1 = new int[]{ 0, 1, 0,-1};
    int[] dr2 = new int[]{ 0, 1, 0,-1};
    int[] dc2 = new int[]{ 1, 0,-1, 0};
    int len = grid.length;
    for (int i=0; i<4; i++) {
      int nr1 = r+dr1[i];
      int nc1 = c+dc1[i];
      int nr2 = r+dr2[i];
      int nc2 = c+dc2[i];
      if (nr1>=0 && nr1<len && nc1>=0 && nc1<len && nr2>=0 && nr2<len && nc2>=0 && nc2<len) {
        if (grid[r][c]==grid[nr1][nc1] && grid[r][c]==grid[nr2][nc2]) {
          return true;
        }
      }
    }
    return false;
  }
}
